/** @module config/state */
import { call, put, takeEvery, select } from 'redux-saga/effects'

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 **/

/** actionTypes define what actions are handeled by the reducer. */
export const actions = {
  configClearForm: () => {
    return {
      type: 'CONFIG/CLEAR_FORM'
    }
  },
  configLoadForm: (key) => {
    return {
      type: 'CONFIG/LOAD_FORM',
      key
    }
  },
  configChangeForm: (data) => {
    return {
      type: 'CONFIG/CHANGE_FORM',
      data
    }
  },
  configGetRequest: () => {
    return {
      type: 'CONFIG/GET_REQUEST'
    }
  },
  configGetSuccess: (data) => {
    return {
      type: 'CONFIG/GET_SUCCESS',
      data
    }
  },
  configGetFailure: error => {
    return {
      type: 'CONFIG/GET_FAILURE',
      error
    }
  },
  configAddRequest: (data) => {
    return {
      type: 'CONFIG/ADD_REQUEST',
      data
    }
  },
  configAddSuccess: (data) => {
    return {
      type: 'CONFIG/ADD_SUCCESS',
      data
    }
  },
  configAddFailure: error => {
    return {
      type: 'CONFIG/ADD_FAILURE',
      error
    }
  },
  configEditRequest: (data) => {
    return {
      type: 'CONFIG/EDIT_REQUEST',
      data
    }
  },
  configEditSuccess: (data) => {
    return {
      type: 'CONFIG/EDIT_SUCCESS',
      data
    }
  },
  configEditFailure: error => {
    return {
      type: 'CONFIG/EDIT_FAILURE',
      error
    }
  },
  configDeleteRequest: (key) => {
    return {
      type: 'CONFIG/DELETE_REQUEST',
      key
    }
  },
  configDeleteSuccess: (data) => {
    return {
      type: 'CONFIG/DELETE_SUCCESS',
      data
    }
  },
  configDeleteFailure: error => {
    return {
      type: 'CONFIG/DELETE_FAILURE',
      error
    }
  },
}
console.log('State actions', actions)

const emptyForm = {
  _id: null,
  key: '',
  description: '',
  value: ''
}

/** state definition */
export const state = {
  configs: [],
  configForm: emptyForm,
  getRequested: false,
  addRequested: false,
  editRequested: false,
  deleteRequested: false
}
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  switch (action.type) {
    case 'CONFIG/CLEAR_FORM':
      return { ...state, configForm: emptyForm }
    case 'CONFIG/LOAD_FORM':
      return { ...state, configForm: { ...state.configs[action.key] } }
    case 'CONFIG/CHANGE_FORM':
      return { ...state, configForm: { ...action.data } }
    case 'CONFIG/GET_SUCCESS':
      return { ...state, configs: action.data.config, getRequested: false }
    case 'CONFIG/GET_FAILURE':
      return { ...state, getRequested: false, alert: action.error }
    case 'CONFIG/ADD_REQUEST':
      return { ...state, addRequested: true }
    case 'CONFIG/ADD_SUCCESS':
      console.log(action)
      return { ...state, configs: [ ...state.configs, action.data ], addRequested: false }
    case 'CONFIG/ADD_FAILURE':
      return { ...state, addRequested: false, alert: action.error }
    case 'CONFIG/EDIT_REQUEST':
      return { ...state, editRequested: true }
    case 'CONFIG/EDIT_SUCCESS':
      const nextConfigs = state.configs.map(config => {
        return config._id == action.data._id ? action.data : config
      })
      return { ...state, configs: nextConfigs, editRequested: false }
    case 'CONFIG/EDIT_FAILURE':
      return { ...state, editRequested: false, alert: action.error }
    case 'CONFIG/DELETE_REQUEST':
      return { ...state, deleteRequested: true }
    case 'CONFIG/DELETE_SUCCESS':
      const newConfigs = [ ...state.configs.slice(0, action.data), ...state.configs.slice(action.data + 1) ] 
      delete newConfigs[action.data.key]
      return { ...state, configs: newConfigs, deleteRequested: false }
    case 'CONFIG/DELETE_FAILURE':
      return { ...state, deleteRequested: false, alert: action.error }
    default:
      return state
  }
}

function * getConfig (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Get config requested', action, token)
    const response = yield call(fetch, 'http://localhost:3002/api/config', {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    }) 
    if (response.status != 200) {
      console.log(response)
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      yield put(actions.configGetSuccess(responseJson))
    }
  } catch (error) {
    console.log('Config failure!', actions.configAddFailure(error))
    yield put(actions.configAddFailure(error))
  }
}

function * addConfig (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Add config requested', action, token)
    const response = yield call(fetch, 'http://localhost:3002/api/config', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      },
      body: JSON.stringify(action.data)
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      yield put(actions.configAddSuccess(action.data))
      yield put(actions.configGetRequest())
    }
  } catch (error) {
    console.log('Config failure!', actions.configAddFailure(error))
    yield put(actions.configAddFailure(error))
  }
}

function * editConfig (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Edit config requested', action, token)
    const response = yield call(fetch, 'http://localhost:3002/api/config', {
      method: 'PUT',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      },
      body: JSON.stringify(action.data)
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      yield put(actions.configEditSuccess(action.data))
    }
  } catch (error) {
    console.log('Config failure!', actions.configEditFailure(error))
    yield put(actions.configEditFailure(error))
  }
}

function * deleteConfig (action) {
  try {
    const token = localStorage.getItem('accessToken')
    const state = yield select()
    const { configs } = state.config
    console.log('Delete config requested', action, token, state, configs)
    const key = configs[action.key].key
    const response = yield call(fetch, 'http://localhost:3002/api/config', {
      method: 'DELETE',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      },
      body: JSON.stringify({ key })
    }) 
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      yield put(actions.configDeleteSuccess(action.key))
    }
  } catch (error) {
    console.log('Config failure!', actions.configDeleteFailure(error))
    yield put(actions.configDeleteFailure(error))
  }
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  console.log('Config saga started.')
  yield takeEvery('CONFIG/GET_REQUEST', getConfig)
  yield takeEvery('CONFIG/ADD_REQUEST', addConfig)
  yield takeEvery('CONFIG/EDIT_REQUEST', editConfig)
  yield takeEvery('CONFIG/DELETE_REQUEST', deleteConfig)
}
